package app.fedilab.fedilabtube.asynctasks;
/* Copyright 2020 Thomas Schneider
 *
 * This file is a part of TubeLab
 *
 * This program is free software; you can redistribute it and/or modify it under the terms of the
 * GNU General Public License as published by the Free Software Foundation; either version 3 of the
 * License, or (at your option) any later version.
 *
 * TubeLab is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
 * the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
 * Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with TubeLab; if not,
 * see <http://www.gnu.org/licenses>. */

import android.content.Context;
import android.os.AsyncTask;

import java.lang.ref.WeakReference;
import java.util.List;

import app.fedilab.fedilabtube.client.PeertubeAPI;
import app.fedilab.fedilabtube.client.entities.Error;
import app.fedilab.fedilabtube.client.entities.Relationship;
import app.fedilab.fedilabtube.interfaces.OnRetrieveRelationshipInterface;


public class RetrieveRelationshipAsyncTask extends AsyncTask<Void, Void, List<Relationship>> {


    private String accountId;
    private OnRetrieveRelationshipInterface listener;
    private Error error;
    private WeakReference<Context> contextReference;

    public RetrieveRelationshipAsyncTask(Context context, String accountId, OnRetrieveRelationshipInterface onRetrieveRelationshipInterface) {
        this.contextReference = new WeakReference<>(context);
        this.listener = onRetrieveRelationshipInterface;
        this.accountId = accountId;
    }

    @Override
    protected List<Relationship> doInBackground(Void... params) {

        PeertubeAPI api = new PeertubeAPI(this.contextReference.get());
        List<Relationship> relationships = api.isFollowing(accountId);
        error = api.getError();
        return relationships;
    }

    @Override
    protected void onPostExecute(List<Relationship> relationships) {
        listener.onRetrieveRelationship(relationships, error);
    }

}
