package app.fedilab.fedilabtube.asynctasks;
/* Copyright 2020 Thomas Schneider
 *
 * This file is a part of TubeLab
 *
 * This program is free software; you can redistribute it and/or modify it under the terms of the
 * GNU General Public License as published by the Free Software Foundation; either version 3 of the
 * License, or (at your option) any later version.
 *
 * TubeLab is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
 * the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
 * Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with TubeLab; if not,
 * see <http://www.gnu.org/licenses>. */

import android.content.Context;
import android.os.AsyncTask;

import java.lang.ref.WeakReference;

import app.fedilab.fedilabtube.client.APIResponse;
import app.fedilab.fedilabtube.client.PeertubeAPI;
import app.fedilab.fedilabtube.interfaces.OnRetrieveAccountsInterface;


public class RetrieveSingleAccountAsyncTask extends AsyncTask<Void, Void, APIResponse> {

    private OnRetrieveAccountsInterface listener;
    private WeakReference<Context> contextReference;
    private String name;
    private actionType type;

    public RetrieveSingleAccountAsyncTask(Context context, String name, actionType type, OnRetrieveAccountsInterface onRetrieveAccountsInterface) {
        this.contextReference = new WeakReference<>(context);
        this.name = name;
        this.listener = onRetrieveAccountsInterface;
        this.type = type;
    }

    @Override
    protected APIResponse doInBackground(Void... params) {
        PeertubeAPI peertubeAPI = new PeertubeAPI(this.contextReference.get());
        if (type == actionType.ACCOUNT) {
            return peertubeAPI.getPeertubeChannel(name);
        } else if (type == actionType.CHANNEL) {
            return peertubeAPI.getPeertubeChannelInfo(name);
        } else {
            return null;
        }
    }

    @Override
    protected void onPostExecute(APIResponse apiResponse) {
        listener.onRetrieveAccounts(apiResponse);
    }

    public enum actionType {
        ACCOUNT,
        CHANNEL
    }

}
